import os
import database_connection as dc
import database_query as dq
from configuration import get_configuration_value


# Where we will store the address translation files
def get_translation_folder():
    return "trans"


# The same folder as get_translation_folder, but less confusing when used
def get_substitution_folder():
    return "trans"


# Get the data name for an adress translation file corresponding to a data_name
def generate_trans_name(data_name):
    return data_name + "_tr"


# Get the data name for an adress translation file corresponding to a data_name
def generate_sub_name(data_name):
    return data_name + "_sub"


def generate_trans_file_name(data_name):
    return generate_trans_name(data_name) + ".atf"


def generate_sub_file_name(data_name):
    return generate_sub_name(data_name) + ".atf"


# Get the full path and filename to an address translation file
def get_trans_file_path(data_name):
    return os.path.join(get_translation_folder(), generate_trans_file_name(data_name))


# Get the full path and filename to an address translation file
def get_sub_file_path(data_name):
    return os.path.join(get_substitution_folder(), generate_sub_file_name(data_name))


def create_tr_sub_database_file(name):
    try:
        connection = dc.get_connection(False, name)
        # This is the base address list, used for downloading the base payload,
        # the data from the actual 10f link
        connection.execute('''CREATE TABLE address
                 (id INTEGER PRIMARY KEY AUTOINCREMENT,
                 pubkey     CHAR,
                 ip         CHAR,
                 port       CHAR
                  );''')
        connection.commit()

        # This is the address list for the Other tr/sub file
        connection.execute('''CREATE TABLE other_tr_sub_address
                 (id INTEGER PRIMARY KEY AUTOINCREMENT,
                 pubkey     CHAR,
                 ip         CHAR,
                 port       CHAR
                  );''')
        connection.commit()
    except:
        # This happens if the files are already there
        pass



def create_address_registry_files(data_name):
    trans_file_path = get_trans_file_path(data_name)
    # print("Create an address registry translation file:", trans_file_path, flush=True)
    ###if file not exists
    create_tr_sub_database_file(trans_file_path)

    sub_file_path = get_sub_file_path(data_name)
    # print("Create an address registry substitution file:", sub_file_path, flush=True)
    create_tr_sub_database_file(sub_file_path)


# dict of addresses for this data_name
# call it with generate_trans_name(data_name) or generate_sub_name(data_name)
# to get the addresses pointing to those two links instead of the base datas links
def get_all_addresses(data_name):
    # todo: flag silent translate in the db for that (-afar) to work

    # get the nodes, having fingers pointing on our data
    data_id = dq.get_link_dataid_by_name(data_name)
    afar = dq.get_afar_flag_from_id(data_id)
    data_addresses = dq.get_address_list_as_dict(data_id, afar)
    return data_addresses


# Gets the addresses out of a file
# Use other_addresses=True to get the other_tr_sub_addresses from a translation file
def load_addresses(file_path_name, other_addresses=False):
    address_list = {}
    if os.path.exists(file_path_name):
        connection = dc.get_connection(False, file_path_name)
        cursor = connection.cursor()
        if other_addresses:
            cursor.execute("SELECT pubkey, ip, port FROM other_tr_sub_address")
        else:
            cursor.execute("SELECT pubkey, ip, port FROM address")

        rows = cursor.fetchall()
        for row in rows:
            pubkey = row[0]
            ip = row[1]
            port = row[2]
            address_list[pubkey] = {'ip': ip, 'port': port}
    else:
        print("load_addresses() could not find file:", file_path_name, flush=True)

    return address_list


# To use with the atf files
# Updates the address part in the 'tr_sub_file_name' file
# by using the other_tr_sub_address in the 'atf_file' file
# Note: the atf file must already have been downloaded
def update_tr_sub_10f_addresses(tr_sub_file_name, atf_file):
    if dc.database_exists():
        print("Check files exist", tr_sub_file_name, atf_file)

        if os.path.exists(tr_sub_file_name) and os.path.exists(atf_file):
            print("Files exist")

            other_addresses = load_addresses(atf_file, other_addresses=True)
            if len(other_addresses):
                # Update the db
                print("Connect to tr/sub file")
                connection = dc.get_connection(False, tr_sub_file_name)
                if connection:
                    print("Connected")
                    connection.execute('''DELETE FROM address_list''')
                    connection.commit()

                    # Insert the new_addresses in trans / sub file
                    for rsa in other_addresses:
                        ip = other_addresses[rsa]['ip']
                        port = other_addresses[rsa]['port']
                        print("Insert ip/port", ip, port)
                        connection.execute('''INSERT INTO address_list (rsa_pub, ip, port) VALUES (?, ?, ?)''',
                                           (rsa, ip, port))
                        connection.commit()


# Watch out, is called with the translation file name
# other_addresses is for the addresses pointing between the _tr and _sub files
def get_addresses_from_translation_file(data_name):
    trans_file_path = get_trans_file_path(data_name)
    address_list = {}
    if os.path.exists(trans_file_path):
        connection = dc.get_connection(False, trans_file_path)
        cursor = connection.cursor()
        cursor.execute("SELECT pubkey, ip, port FROM address")

        rows = cursor.fetchall()
        for row in rows:
            pubkey = row[0]
            ip = row[1]
            port = row[2]
            address_list[pubkey] = {'ip': ip, 'port': port}

    return address_list


# todo: refact this by using load_addresses() instead
# Call with full filepath name
def get_other_addresses_from_tr_sub_file(file_path_name):
    address_list = {}
    if os.path.exists(file_path_name):
        connection = dc.get_connection(False, file_path_name)
        cursor = connection.cursor()
        cursor.execute("SELECT pubkey, ip, port FROM other_tr_sub_address")

        rows = cursor.fetchall()
        for row in rows:
            pubkey = row[0]
            ip = row[1]
            port = row[2]
            address_list[pubkey] = {'ip': ip, 'port': port}

    return address_list


# Checks if the address translation file contain exactly these addesses (for the base link file), else return False
def verify_addres_database(data_name):
    existing_addresses = get_addresses_from_translation_file(data_name)
    new_addresses = get_all_addresses(data_name)

    # Verify there are the same addresses in the trans file
    return existing_addresses == new_addresses


# if check_tr then
# check _tr file vs _sub file
# else _sub file vs _tr file
def verify_tr_sub_addresses(data_name, check_tr):
    if check_tr:
        existing_addresses = get_other_addresses_from_tr_sub_file(get_trans_file_path(data_name))
    else:
        existing_addresses = get_other_addresses_from_tr_sub_file(get_sub_file_path(data_name))

    new_addresses = get_all_addresses(data_name)

    # Verify there are the same addresses
    return existing_addresses == new_addresses


# Reset an address translation file with its addresses, you should update it as the file have changed! (like ./10f.exe -u)
# Note: YOU must update version of the file, this is a raw, no checks update.
# if use_tr_file==False, then it updates the sub file
def set_addresses(data_name, use_tr_file):
    if use_tr_file:
        tr_sub_file_name = get_trans_file_path(data_name)  # the .atf file
        other_name = generate_sub_name(data_name)
    else:
        tr_sub_file_name = get_sub_file_path(data_name)
        other_name = generate_trans_name(data_name)

    if os.path.exists(tr_sub_file_name):
        # Get all addresses from the base data
        new_addresses = get_all_addresses(data_name)
        other_addresses = get_all_addresses(other_name)

        # Clean out trans file
        # print("Clean out address from", tr_sub_file_name, flush=True)
        connection = dc.get_connection(False, tr_sub_file_name)
        if connection:
            connection.execute('''DELETE FROM address''')
            connection.commit()
            connection.execute('''DELETE FROM other_tr_sub_address''')
            connection.commit()

            # Insert the new_addresses in trans file
            for rsa in new_addresses:
                ip = new_addresses[rsa]['ip']
                port = new_addresses[rsa]['port']
                connection.execute('''INSERT INTO address (pubkey, ip, port) VALUES (?, ?, ?)''', (rsa, ip, port))
                connection.commit()

            for rsa in other_addresses:
                ip = other_addresses[rsa]['ip']
                port = other_addresses[rsa]['port']
                connection.execute('''INSERT INTO other_tr_sub_address (pubkey, ip, port) VALUES (?, ?, ?)''', (rsa, ip, port))
                connection.commit()


# Updates tr & sub files according to what kind of data this is (base_data, TR, SUB)
def refresh_data_addresses(data_id):
    # If this is a base_data, udate the corresponding TR&SUB files, else update the 'other' SUB/TR file
    from insert_data import insert_our_data

    data_type = dq.get_translation_file_type(data_id)
    my_name, _ = dq.get_link_data_by_dataid(data_id)

    update_trans_file = False
    update_sub_file = False
    # Keep code simple here
    if data_type == 0:  # base data, update both TR&SUB
        update_trans_file = True
        update_sub_file = True
    if data_type == 1:
        # We got a new node linking to our TR data, Update corresponding SUB file
        update_sub_file = True
    if data_type == 2:
        # We got a new node linking to our SUB data, Update corresponding TR file
        update_trans_file = True

    if update_trans_file:
        set_addresses(my_name, use_tr_file=True)
        # The (translation) file to reincrypt
        trans_name = generate_trans_name(my_name)
        trans_file_name = generate_trans_file_name(my_name)  # name => name_tr.atf
        # update_data_main(['', trans_name, ar.get_translation_folder(), trans_file_name])

        insert_our_data(trans_name, get_translation_folder(), trans_file_name)  # is actually create OR update ...

    if update_sub_file:
        set_addresses(my_name, use_tr_file=False)
        # The (substitute) file to reincrypt
        sub_name = generate_sub_name(my_name)
        sub_file_name = generate_sub_file_name(my_name)  # name => name_sub.atf
        # update_data_main(['', sub_name, ar.get_translation_folder(), sub_file_name])
        insert_our_data(sub_name, get_translation_folder(), sub_file_name)  # is actually create OR update ...
















